<?php

// Random Image widget
//
// Copyright (c) 2008 Marcel Proulx
// http://www.district30.net/random-image/
//
// This is an add-on for WordPress
// http://wordpress.org/
//
// **********************************************************************
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// *****************************************************************

/*
Plugin Name: Random Image Widget
Plugin URI: http://www.district30.net/random-image/
Description: Display a random image from a directory located on the webserver
Author: Marcel Proulx
Version: 1.2
Author URI: http://www.district30.net
*/ 

/* Function: widget_random_image_init
** This function contains all of the functions we use to create the widget; this
** ensures that everything is loaded.
**
** args: none
** returns: nothing
*/

function widget_random_image_init() {

	if(!function_exists('register_sidebar_widget'))
		return;

	/* Function: disp_random_image
	** This function does the actual display of the widget
	**
	** args: environment variables (handled automatically by the hook)
	** returns: nothing
	*/

	function disp_random_image($args,$number) {

		// Each widget can store its own options. We keep strings here.
		$options = get_option('widget_random_image');
		$ri_title = $options[$number]['title'];
		$ri_image_count = $options[$number]['image_count'];
		$ri_image_directory = $options[$number]['directory'];
		$ri_recurse = $options[$number]['recurse'];
		$ri_show_caption = $options[$number]['show_caption'];
		$ri_caption = $options[$number]['caption'];
		$ri_show_link = $options[$number]['show_link'];
		$ri_link = $options[$number]['link'];
		
		//get the required environment variables and begin to show the widget
		extract($args);

		echo $before_widget;
		echo $before_title
			. $ri_title
			. $after_title;

        echo '<div id="random_image_wrapper" align="center">';

		//read the directory and return a list of files
		
		$path = get_bloginfo('wpurl');
		$image_directory = ABSPATH . $ri_image_directory;
		$images = image_list($image_directory,$path . $ri_image_directory, $ri_recurse);
		$image_count = sizeof($images);
		
		usort($images, ri_rand_compare);
		usort($images, ri_rand_compare); //if you do it twice, it's twice as random, right?  Seemed like the last image came up too much.

  		for ($count = 0; $count < $ri_image_count; $count ++) {
		
			$item_url = $images[$count];
			//perform scaling
            $img_info = @getimagesize($item_url);  //image width is in element 0 and height is element 1
			$img_width = $img_info[0];
			$img_height = $img_info[1];

			if ($img_width < $img_height) {
				//this is probably a "portrait" image so we'll scale vertically to 150 pixels
				$img_scale = ' height="150"';
			}
			else {
				//otherwise, this is probably a "landscape" image so we'll scale horizontally to 150 pixels
            	$img_scale = ' width="150"';
			}

			//generate the appropriate code to show the photo
			echo '<a href="';
			echo $ri_show_link ? $ri_link : $item_url;
			echo '"><img style = "padding: 3px 3px 3px 3px;" src="' . $item_url . '"';
			echo $img_scale;
			echo ' alt="' . $ri_caption . '"';
			echo ' /></a>';
			echo "\n";
			echo $ri_show_caption ? '<p>' . $ri_caption . '</p>' : '';
		}

		//finish sidebar code
		echo '</div>'; //an if statement isn't necessary since we opened a div regardless of whether a border was selected
		echo $after_widget;

	}	

    /* Function: widget_random_image_register
	**
	**  This function registers the sidebar widget and its control form.
	**	The function checks the options to see how many instances are desired
	**
	** args: none
	** returns: none
	*/
	
	function widget_random_image_register() {
 		if ( isset($_POST['image-number-submit']) ) {
			$options=get_option('widget_random_image');
			$number = (int) $_POST['image-number'];
			if ( $number > 9 ) $number = 9;
			if ( $number < 1 ) $number = 1;
			$options['instances']=$number;
			update_option('widget_random_image',$options);
		}
		
		//This registers the widget with the Sidebar plugin
		$options=get_option('widget_random_image'); //refresh options
		$sb_instances = (int) $options['instances'];
		if ($sb_instances < 1) $sb_instances = 1;
		if ($sb_instances > 9) $sb_instances = 9; //these conditions shouldn't happen, but just in case, we'll limit instances here
		for ($i = 1; $i<=9; $i++) {
			$name = sprintf(__('Random Image %d'), $i);
			$id = "random_image-$i"; // Never never never translate an id
			$dims = array('width' => 550, 'height' => 275);
			$class = array('classname'=>'random_image');

			wp_register_sidebar_widget($id, $name, $i <= $sb_instances ? 'disp_random_image' : /* unregister */ '', $class, $i);
			wp_register_widget_control($id, $name, $i <= $sb_instances ? 'widget_random_image_control' : /*unregister*/ '', $dims, $i);

		}
 		
	}
	
	widget_random_image_register();
	
 	/* Function: ri_rand_compare
	** To be used with an array sort function (i.e. usort), this function 
	** will randomly sort an array
	**
	** args: $x=string; $y=string (handled by the function call)
	** returns: a random integer between -1 and 1
	*/

	function ri_rand_compare($xt, $yt) {
 		// We don't care about the array elements passed on so we'll
		// create a random value for the return value
		return mt_rand(-1,1);
	}

 	/* Function: image_list
	** This function recursively searches directories for image files 
	** Because these are accessed via their path on the server
	** but we want to return URL's (based on the web root), both paths
	** are required arguments.
	**
	** args: $image_directory=string; $image_url=string; $recursive=boolean
	** returns: an array containing image filenames
	*/

	function image_list($image_directory,$image_url,$recursive=FALSE) { 
		$images = array();
		$dirhandle = opendir($image_directory);
		while (($filename = readdir($dirhandle))!== FALSE) {
			if (strcmp($filename, '.') == 0 || strcmp($filename, '..') == 0) continue;  //skip the rest of the while loop if we're looking at . or ..
			$abs_filename = $image_directory . '/' . $filename;
			$file_extention=strtolower(substr($filename,strrpos($filename, '.')+1));
			if ($file_extention == 'jpg' or $file_extention == 'gif') {
				$images[] = $image_url . '/' . $filename;
			}
			elseif (is_dir($abs_filename) && $recursive) {
				$images = array_merge($images,image_list($abs_filename, $image_url . '/' . $filename));
			}
		}
		closedir($dirhandle);
		return $images;
	}
	
	/* Function: widget_random_image_control()
	** This is the function that outputs the form to let the users edit
	** the widget's title, the number of photos to display and the location
	** of photos;
	**
	** args: instance number(int)
	** returns: nothing
	*/

	function widget_random_image_control($number) {

		// Get our options and see if we're handling a form submission.
		$options = get_option('widget_random_image');
		
		//if this is the first time opening the admin page after the plugin has been updated from 1.1 or ealier, update the database
		if (!is_array($options[1]) and is_array($options)) {
			update_random_image_options();
			$options = get_option('widget_random_image'); //reload options after updating database
		}
		
		if ( !is_array($options[$number]) ) {
			$options[$number] = array('title'=>'Random', 'image_count'=>'1', 'directory'=>'/wp-content/plugins/random-image', 'recurse'=>FALSE, 'show_caption'=>FALSE, 'caption'=>'', 'show_link'=>FALSE, 'link'=>'');

		}
		if ( $_POST['random_image-submit'] ) {
            // Remember to sanitize and format use input appropriately.
			$options[$number]['title'] = strip_tags(stripslashes($_POST["random_image-title-$number"]));
   			$options[$number]['image_count'] = strip_tags(stripslashes($_POST["random_image-image_count-$number"]));
			$options[$number]['directory'] = strip_tags(stripslashes($_POST["random_image-directory-$number"]));
			$options[$number]['recurse'] = strip_tags(stripslashes($_POST["random_image-recurse-$number"]));
			$options[$number]['show_caption'] = strip_tags(stripslashes($_POST["random_image-show_caption-$number"]));
			$options[$number]['caption'] = strip_tags(stripslashes($_POST["random_image-caption-$number"]));
			$options[$number]['show_link'] = strip_tags(stripslashes($_POST["random_image-show_link-$number"]));
			$options[$number]['link'] = strip_tags(stripslashes($_POST["random_image-link-$number"]));
			update_option('widget_random_image', $options);
		}

		// Be sure you format your options to be valid HTML attributes.
		$control_ri_title = htmlspecialchars($options[$number]['title'], ENT_QUOTES);
		$control_ri_image_count = htmlspecialchars($options[$number]['image_count'], ENT_QUOTES);
		$control_ri_image_directory = htmlspecialchars($options[$number]['directory'], ENT_QUOTES);
		$control_ri_recurse = htmlspecialchars($options[$number]['recurse'], ENT_QUOTES);
		$control_ri_show_caption = htmlspecialchars($options[$number]['show_caption'], ENT_QUOTES);
		$control_ri_caption = htmlspecialchars($options[$number]['caption'], ENT_QUOTES);
		$control_ri_show_link = htmlspecialchars($options[$number]['show_link'], ENT_QUOTES);
		$control_ri_link = htmlspecialchars($options[$number]['link'], ENT_QUOTES);
  		
  		// Here is our little form segment. Notice that we don't need a
		// complete form. This will be embedded into the existing form.

		echo '<p style="text-align:left; line-height=1.2em"><label for="random_image-title-' . $number . '">Title:  ' . ' </label><input style="width: 200px;" id="random_image-title-' . $number . '" name="random_image-title-' . $number . '" type="text" value="'.$control_ri_title.'" /></p>';
  		//create an option box with a set number of values generated by a for loop
		echo '<p style="text-align:left; line-height=1.2em"><label for="random_image-image_count-' . $number . '">' . 'Number of images to show:  ' . '</label><select name="random_image-image_count-' . $number . '" id="random_image-image_count-' . $number . '" style="width: 50px;">';
		for($list_val=1; $list_val <= 10; $list_val ++) {
			if ($list_val == $control_ri_image_count)
				echo '<option selected>' . $list_val . '</option>';
			
			else 
				echo '<option>' . $list_val . '</option>';
		}
		echo '</select></p>';
		echo '<p style="text-align:left; line-height=1.2em"><label for="random_image-directory-' . $number . '">' . 'Path to image directory:  ' . '</label><input style="width: 350px; margin-left: 10;" id="random_image-directory-' . $number . '" name="random_image-directory-' . $number . '" type="text" value="' . $control_ri_image_directory . '" /></p>';
		echo '<p style="text-align:left; line-height=1.2em"><input style="margin-left: 10;" id="random_image-recurse-' . $number . '" name="random_image-recurse-' . $number . '" type="checkbox" value="1" ';
		echo  $control_ri_recurse ? 'checked' : ''; //if the option was selected, make the box checked when the form is open; otherwise leave it unchecked
		echo '>';
		echo  '</input><label for="random_image-recurse-' . $number . '">' . ' Look for photos in subdirectories  ' . '</label></p>';
		echo '<p style="text-align:left; line-height=1.2em"><input style="margin-left: 10;" id="random_image-show_caption-' . $number . '" name="random_image-show_caption-' . $number . '" type="checkbox" value="1" ';
		echo  $control_ri_show_caption ? 'checked' : ''; //if the option was selected, make the box checked when the form is open; otherwise leave it unchecked
		echo '>';
		echo  '</input><label for="random_image-show_caption-' . $number . '">' . ' Show caption beneath photo: ' . '</label>';
		echo ' <input style="width: 300px; margin-right: 20" id="random_image-caption-' . $number . '" name="random_image-caption-' . $number . '" type="text" value="' . $control_ri_caption . '"/></p>';
		
		echo '<p style="text-align:left; line-height=1.2em"><input style="margin-left: 10;" id="random_image-show_link-' . $number . '" name="random_image-show_link-' . $number . '" type="checkbox" value="1" ';
		echo  $control_ri_show_link ? 'checked' : ''; //if the option was selected, make the box checked when the form is open; otherwise leave it unchecked
		echo '>';
		echo  '</input><label for="random_image-show_link-' . $number . '">' . ' Link to URL: ' . '</label>';
		echo ' <input style="width: 408px; margin-right: 20;" id="random_image-link-' . $number . '" name="random_image-link-' . $number . '" type="text" value="' . $control_ri_link . '"/></p>';
		
   		echo '<input type="hidden" id="random_image-submit" name="random_image-submit" value="1" />';
    }
    
    /* Function: widget_random_image_page()
	** This is the function that outputs the form to let the users edit
	** the widget's global settings, (i.e., the number of plugin instances)
	** Minor adjustments from the built-in Text and RSS widget control pages.
	** args: none
	** returns: nothing
	*/

	function widget_random_image_page() {
	$options = $newoptions = get_option('widget_random_image');

?>
	<div class="wrap">
		<form method="POST">
			<h2><?php _e('Image Widgets'); ?></h2>
			<p style="line-height: 30px;"><?php _e('How many random image widgets would you like?'); ?>
			<select id="image-number" name="image-number" value="<?php echo $options['instances']; ?>">
<?php for ( $i = 1; $i < 10; ++$i ) echo "<option value='$i' ".( $options['instances']==$i ? "selected='selected'" : '').">$i</option>"; ?>
			</select>
			<span class="submit"><input type="submit" name="image-number-submit" id="image-number-submit" value="<?php echo attribute_escape(__('Save')); ?>" /></span></p>
		</form>
	</div>
<?php
	}

	/* Function: update_photo_options()
	** Updates version 1.1 or earlier settings into higher-version data 
	** structure by moving old settings into the 1st "instance" of the plugin.  
	** Should only be run the first time the Widget Admin page is opened.
	** args: none
	** returns: nothing
	*/

	function update_random_image_options() {
	
	$options = get_option('widget_random_image');

	$new_options[1]['title'] = $options['title'];
	$new_options[1]['image_count'] = $options['image_count'];
	$new_options[1]['directory'] = $options['directory'];
	
	//modify the directory option to make it suitable for the new version 
	if (substr($new_options[1]['directory'],0,1) <> '/')
		$new_options[1]['directory'] = '/' . $new_options[1]['directory']; //add a '/' at the beginning
	if (substr($new_options[1]['directory'],-1,1) == '/')
		$new_options[1]['directory'] = substr($new_options[1]['directory'],0,-1); //return everything but the last '/'
	
	delete_option('widget_random_image');
	update_option('widget_random_image',$new_options);

	}

}

add_action('widgets_init','widget_random_image_init');
add_action('sidebar_admin_page','widget_random_image_page');

?>
